#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar 11 08:07:21 2021
Caractérisation de la variabilité sur des essais expérimentaux
@author: xavierjourdain
"""

# Bibliothèque de calcul scientifique
import numpy as np
# Bibliothèque pour réaliser des graphiques
import matplotlib.pyplot as plt
# Bibliothèque d'outils statistiques (dont lognorm pour la loi log-normale)
from scipy import stats

#### FONCTIONS ###############################################
# Fonction loi de probabilité = loi log normale
def loi_log_normale(x_tmp,moyenne_tmp,ecart_type_tmp):
    #log_normale_eq = 1/(x_tmp * ecart_type_tmp * np.sqrt(2*np.pi)) * np.exp(-(np.log(x_tmp)-moyenne_tmp)**2 / (2*ecart_type_tmp**2))
    # fonction scipy : stats.lognorm.pdf(x , s, loc, scale)  <=> lognorm.pdf(y, s) / scale with y = (x - loc) / scale
    log_normale_scipy = stats.lognorm.pdf(x_tmp,s=ecart_type_tmp,loc=0,scale=np.exp(moyenne_tmp))
    return log_normale_scipy

##### DONNÉES À DÉFINIR PAR L'UTILISATEUR ####################

# Import du fichier de données
nom_fichier = 'Resultats_Exp_Traction_Fmax'
donnees_exp = np.genfromtxt(''+nom_fichier+'.csv', delimiter=',')
# Nombre d'intervalles réguliers pour le découpage de l'histogramme
nb_intervalles_histo = 20

#### ANALYSE DES DONNÉES #####################################
# Caractérisation des résultats expérimentaux
esperance = np.average(donnees_exp)
print("espérance = ",esperance)
ecart_type = np.std(donnees_exp,ddof=0)
print("écart type avec biais = ",ecart_type)
ecart_type = np.std(donnees_exp,ddof=1)
print("écart type sans biais = ",ecart_type)
# Écart-type et Moyenne pour la loi lognormale
ecart_type_lognorm = np.sqrt(np.log(1+ecart_type**2/esperance**2))
moyenne_lognorm = np.log(esperance) - 0.5 * ecart_type_lognorm**2
minimum = np.min(donnees_exp)
maximum = np.max(donnees_exp)
if (maximum - minimum) <= maximum:
    print("NB : Potentielles erreurs de 'float' (min = {}, max = {}, delta = {} < maximum)".format(minimum,maximum,maximum-minimum))
nb_donnees = len(donnees_exp)
largeur_intervalles_histo = (maximum-minimum)/nb_intervalles_histo

# Histogramme des résultats expérimentaux
fig1 = plt.figure()
plt.title("Répartition des {} résultats expérimentaux\n(en nombre)".format(nb_donnees))
fig1.patch.set_alpha(0.0)
plt.hist(donnees_exp,bins=nb_intervalles_histo,histtype='stepfilled', label="données expérimentales")
plt.xlabel('Effort max [N]') 
plt.ylabel('Nombre de résultats par intervalle') 
plt.legend()
plt.show()

fig1 = plt.figure()
plt.title("Répartition des {} résultats expérimentaux\n(intégrale à 1)".format(nb_donnees))
fig1.patch.set_alpha(0.0)
[valeurs_y_donnees_exp,lim_bins,rectangles] = plt.hist(donnees_exp,bins=nb_intervalles_histo,histtype='stepfilled',density=True,label="données expérimentales")
plt.xlabel('Effort max [N]') 
plt.ylabel("probabilité d'occurence/\n((max-min)/nb_intervalles)") 
plt.legend()
plt.show()

#### COMPARAISON DONNÉES EXP/MODÈLE ##########################

# Valeurs pour la loi log-normale identifiée
valeurs_x = np.linspace(minimum,maximum,100)
valeurs_y_loi_log_normale = loi_log_normale(valeurs_x,moyenne_lognorm,ecart_type_lognorm)
fig1 = plt.figure()
plt.title("loi log-normale identifiée".format(nb_donnees))
fig1.patch.set_alpha(0.0)
plt.plot(valeurs_x, valeurs_y_loi_log_normale,'r--', label='loi log-normale identifiée')
plt.xlabel('Effort max [N]') 
plt.ylabel("probabilité d'occurrenece") 
plt.legend()
plt.show()


fig1 = plt.figure()
plt.title("Fonctions de répartition de la loi identifiée\n(moyenne = {} N ; écart-type = {} N)".format(round(esperance,1),round(ecart_type,1)))
valeurs_x2 = np.linspace(0,1.,100)
plt.plot(stats.lognorm.ppf(valeurs_x2, s=ecart_type_lognorm,loc=0,scale=np.exp(moyenne_lognorm)),valeurs_x2 ,'r-', lw=2, alpha=0.6, label="loi lognormale")
plt.legend()
plt.show()

# Calcul de l'erreur loi / données
x_interv_histo = np.linspace(minimum,maximum,nb_intervalles_histo)
valeurs_y_histo_loi_log_normale = loi_log_normale(x_interv_histo,moyenne_lognorm,ecart_type_lognorm)
erreur = np.linalg.norm(valeurs_y_histo_loi_log_normale - valeurs_y_donnees_exp)
erreur_relative = erreur / sum(valeurs_y_donnees_exp)

#### RÉCUPÉRATION VALEUR FONCTION QUANTILE #################

pourcentage = 0.05
valeur_cible = round(stats.lognorm.ppf(pourcentage,s=ecart_type_lognorm,loc=0,scale=np.exp(moyenne_lognorm)),1)

# Graphique non normé (intégrale = nb d'essais)
fig1 = plt.figure()
plt.title("Répartition des {} résultats expérimentaux\n(en nombre)\n[erreur relative : {}%]".format(nb_donnees,round(100*erreur_relative,2)))
fig1.patch.set_alpha(0.0)
plt.plot(valeurs_x, valeurs_y_loi_log_normale * nb_donnees * (maximum-minimum)/nb_intervalles_histo,'r--', label='loi log-normale identifiée')
plt.hist(donnees_exp,bins=nb_intervalles_histo,histtype='stepfilled', label="données expérimentales")
plt.axvline(x=valeur_cible,color='green')
plt.xlabel('Effort max [N]') 
plt.ylabel('Nombre de résultats par intervalle') 
plt.legend()
plt.show()

# Graphique normé (intégrale = 1)
fig1 = plt.figure()
plt.title("Répartition des {} résultats expérimentaux\n(intégrale à 1)\n[erreur relative : {}%]".format(nb_donnees,round(100*erreur_relative,2)))
fig1.patch.set_alpha(0.0)
plt.plot(valeurs_x, valeurs_y_loi_log_normale,'r--', label='loi lognormale identifiée')
plt.hist(donnees_exp,bins=nb_intervalles_histo,histtype='stepfilled',density=True, label="données expérimentales")
plt.axvline(x=valeur_cible,color='green',label="quantile {}% : {}".format(pourcentage*100,valeur_cible))
plt.xlabel('Effort max [N]') 
plt.ylabel("probabilité d'occurence/\n((max-min)/nb_intervalles)") 
plt.legend()
plt.show()

# Sortie des valeurs souhaitées
print("Étude sur {} valeurs expérimentales :".format(nb_donnees))
print("{}% des valeurs de la loi lognormale identifiée sont au dessus de {} N".format(100*(1-pourcentage),valeur_cible))
